/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.voxreader;

import fr.amap.lidar.amapvox.commons.Voxel;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import org.apache.log4j.Logger;

/**
 *
 * @author calcul
 */
public class VoxelFileReader extends AbstractReader implements Iterable<Voxel> {

    private final static Logger LOGGER = Logger.getLogger(VoxelFileReader.class);
    private final int[] nwarning;
    private final int N_WARNING_MAX = 50;

    public VoxelFileReader(File voxelFile, boolean keepInMemory) throws Exception {

        super(voxelFile, keepInMemory);
        nwarning = new int[voxelSpace.getVoxelSpaceInfos().getColumnNames().length];
    }

    public VoxelFileReader(File voxelFile) throws Exception {

        super(voxelFile);
        nwarning = new int[voxelSpace.getVoxelSpaceInfos().getColumnNames().length];
    }

    public static boolean isFileAVoxelFile(File file) {

        try (BufferedReader reader = new BufferedReader(new FileReader(file))) {

            String identifier = reader.readLine();

            if (!identifier.equals("VOXEL SPACE")) {
                reader.close();
                return false;
            }

            reader.close();

            return true;

        } catch (FileNotFoundException ex) {
            return false;
        } catch (IOException ex) {
            return false;
        }
    }

    @Override
    public Iterator<Voxel> iterator() {

        currentVoxelIndex = -1;

        if ((!wasRead && keepInMemory) || !keepInMemory) {

            try {
                reader = new BufferedReader(new FileReader(voxelFile));

                for (int i = 0; i < 6; i++) {
                    reader.readLine();
                }

            } catch (FileNotFoundException ex) {
                LOGGER.error("Cannot find file", ex);
            } catch (IOException ex) {
                LOGGER.error("Cannot read file", ex);
            }
        }

        Iterator<Voxel> it;

        it = new Iterator<Voxel>() {

            @Override
            public boolean hasNext() {

                if ((!wasRead && keepInMemory) || !keepInMemory) {
                    try {
                        boolean isNextExist = ((currentLine = reader.readLine()) != null);

                        if (!isNextExist) {
                            reader.close();
                            wasRead = true;
                        }

                        return isNextExist;
                    } catch (IOException ex) {
                        return false;
                    }

                } else {
                    return voxelSpace.voxels != null && currentVoxelIndex + 1 < voxelSpace.voxels.size();
                }
            }

            @Override
            public Voxel next() {

                //on parse la ligne
                currentVoxelIndex++;

                Voxel voxel;

                if (wasRead) {
                    if (keepInMemory) {
                        return (Voxel) voxelSpace.voxels.get(currentVoxelIndex);
                    } else {
                        voxel = parseVoxelFileLine(currentLine);
                    }
                } else {

                    voxel = parseVoxelFileLine(currentLine);

                    if (voxel == null) {
                        StringBuilder sb = new StringBuilder();
                        sb.append("Voxel file ")
                                .append(voxelFile.getName())
                                .append(" error parsing line ")
                                .append((currentVoxelIndex + 7));
                        LOGGER.error(sb.toString());
                    } else if (keepInMemory) {
                        voxelSpace.voxels.add(voxel);
                    }

                }

                return voxel;
            }
        };

        return it;
    }

    public Voxel parseVoxelFileLine(String line) {

        try {

            String[] voxelLine = line.split(" ");

            Voxel vox = new Voxel(
                    Integer.valueOf(voxelLine[0]), // x
                    Integer.valueOf(voxelLine[1]), // y
                    Integer.valueOf(voxelLine[2])); // z

            int ncolumn = voxelSpace.getVoxelSpaceInfos().getColumnNames().length;
            for (int i = 3; i < ncolumn; i++) {
                try {
                    vox.setFieldValue(vox.getClass(), voxelSpace.getVoxelSpaceInfos().getColumnNames()[i], vox, voxelLine[i]);
                } catch (NoSuchFieldException | SecurityException | IllegalArgumentException | IllegalAccessException ex) {
                    nwarning[i] += 1;
                    StringBuilder sb;
                    if (nwarning[i] < N_WARNING_MAX) {
                        sb = new StringBuilder();
                        sb.append("Voxel file ")
                                .append(voxelFile.getName())
                                .append(" line ")
                                .append(currentVoxelIndex + 7)
                                .append(" column ")
                                .append(getVoxelSpaceInfos().getColumnNames()[i])
                                .append('\n');
                        sb.append("Failed to parse value ")
                                .append(voxelLine[i])
                                .append(".");
                        LOGGER.warn(sb, ex);
                    } else if (nwarning[i] == N_WARNING_MAX) {
                        sb = new StringBuilder();
                        sb.append("Voxel file ")
                                .append(voxelFile.getName())
                                .append(" column ")
                                .append(getVoxelSpaceInfos().getColumnNames()[i])
                                .append(": two many warnings.").append('\n')
                                .append("The whole columns may be flawed. Will not show further warning for this column.");
                        LOGGER.warn(sb);
                    }
                }
            }
            return vox;

        } catch (Exception e) {
            return null;
        }

    }
}
