package fr.amap.lidar.amapvox.gui;

import java.net.URL;
import java.net.URLClassLoader;
import java.util.Locale;
import java.util.ResourceBundle;
import javafx.application.Application;
import javafx.collections.ObservableList;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.stage.Screen;
import javafx.stage.Stage;
import javafx.stage.WindowEvent;

public class JavafxApp extends Application {

    @Override
    public void start(Stage stage) throws Exception {

        ResourceBundle rb = ResourceBundle.getBundle("bundle_help", 
                Locale.ENGLISH, new URLClassLoader(new URL[]{getClass().getResource("/strings/")}));
        FXMLLoader loader = new FXMLLoader(getClass().getResource("/fxml/MainFrame.fxml"), rb);
        Parent root = loader.load();
        MainFrameController controller = loader.getController();
        ObservableList<Screen> screens = Screen.getScreensForRectangle(0, 0, 10, 10);

        if (screens != null && screens.size() > 0) {
            stage.setWidth(screens.get(0).getBounds().getWidth());
            stage.setHeight(screens.get(0).getBounds().getHeight());
        }

        Scene scene = new Scene(root);
        scene.getStylesheets().add("/styles/Styles.css");

        stage.setTitle("AMAPVox " + Global.buildVersion);
        stage.setScene(scene);
        stage.getIcons().addAll(
                new Image("/icons/icon_256x256.png"),
                new Image("/icons/icon_128x128.png"),
                new Image("/icons/icon_64x64.png"));

        controller.setStage(stage);

        stage.addEventFilter(WindowEvent.WINDOW_HIDDEN, (WindowEvent event) -> {
            System.exit(0);
        });

        stage.show();
    }
}