/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization;

import fr.amap.commons.util.debug.DebugLevel;
import fr.amap.commons.util.debug.Debugger;
import fr.amap.commons.util.swing.jprogress.JProgressFrame;
import fr.amap.lidar.amapvox.visualization.views.VisualizationScene;
import java.io.InputStream;
import fr.amap.lidar.amapvox.visualization.views.MainView;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.UnsupportedLookAndFeelException;

/**
 * Main class of the Voxelization project.
 */
public class Visualization {

    /**
     * Visualization function of the project (only used to testing)
     * @param args Arguments (not needed)
     */
    public static void main(String[] args) {
        launch(Visualization.class.getResourceAsStream("/data/test.vox"), "/data/test.vox");
    }

    /**
     * Commodity function to launch a visualization application without default
     * property.<br>
     * Work exactly as {@code launch(in, null);}
     *
     * @param in Voxel space file to visualize.
     * @param filepath
     */
    public static final void launch(final InputStream in, final String filepath) {
        launch(in, filepath, null);
    }

    /**
     * Function used to launch a Visualization application.
     *
     * @param in Voxel space file to visualize.
     * @param filepath
     * @param defaultProperty The default voxel property to display.
     */
    public static final void launch(final InputStream in, final String filepath, 
            final String defaultProperty) {
        /* Set the Nimbus look and feel */
        try {
            javax.swing.UIManager.
                    setLookAndFeel("javax.swing.plaf.nimbus.NimbusLookAndFeel");
        } catch (ClassNotFoundException | InstantiationException
                | IllegalAccessException | UnsupportedLookAndFeelException ex) {
            Logger.getLogger(Visualization.class.getName()).log(Level.SEVERE, null, ex);
        }

        javax.swing.SwingUtilities.invokeLater(() -> {
            final JProgressFrame progressFrame = new JProgressFrame();
            progressFrame.setProgression(1, 3, "Generate the scene...");
            progressFrame.init();

            new Thread(() -> {
                try {
                    final MainView mainView = new MainView();
                    final VisualizationScene scene = new VisualizationScene();

                    progressFrame.setProgression(2, 3, "Import the voxel space file...");
                    scene.setVoxelSpaceFile(in, filepath, progressFrame);

                    progressFrame.setProgression(3, 3, "Last process...");
                    mainView.setScene(scene);
                    
                    if (defaultProperty != null) {
                        scene.getVoxelSpaceComponent().setProperty(defaultProperty);
                        mainView.setSelectedProperty(defaultProperty);
                    }

                    progressFrame.dispose();
                    mainView.setVisible(true);
                } catch (Exception e) {
                    Debugger.debugInterface(e, DebugLevel.ERROR, null);
                }
            }).start();
        });
    }

}
